package com.fr.plugin.login.handler;

import com.fr.decision.fun.impl.BaseHttpHandler;
import com.fr.json.JSONObject;
import com.fr.plugin.login.config.simple.account.PluginSimpleConfig;
import com.fr.plugin.login.utils.FRUtils;
import com.fr.plugin.login.utils.ResponseUtils;
import com.fr.plugin.login.utils.Utils;
import com.fr.plugin.transform.FunctionRecorder;
import com.fr.plugin.weixin.server.log.WeiXinLogErrorCode;
import com.fr.plugin.weixin.server.log.WeiXinLoggerFactory;
import com.fr.plugin.weixin.server.util.WeiXinRequest;
import com.fr.plugin.weixin.server.util.WeiXinUserUtils;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@FunctionRecorder
public class WxAuth extends BaseHttpHandler {


    public WxAuth() {
    }

    @Override
    public RequestMethod getMethod() {
        return RequestMethod.GET;
    }

    @Override
    public String getPath() {
        return "/wxAuth";
    }

    @Override
    public boolean isPublic() {
        return true;
    }

    @Override
    public void handle(HttpServletRequest req, HttpServletResponse res) throws Exception {
        PluginSimpleConfig psc = PluginSimpleConfig.getInstance();
        //登录页面登陆的用户名
        String username = req.getParameter("username");
        String code = req.getParameter("code");
        String secret = psc.getSecret();
        String realUsername =weixinAuth(code,secret);

        if(Utils.isNullStr(realUsername)){
            ResponseUtils.failedResponse(res,"微信鉴权失败，请联系管理员!");
            return;
        }

        if(!username.equals(realUsername)){
            ResponseUtils.failedResponse(res,"微信关联账户与登录账户不一致，请重新登录!");
            return;
        }
        String index = psc.getIndex();

        FRUtils.login(req,res,username,index);
    }

    private static String weixinAuth(String code,String secret) throws Exception{
        JSONObject userInfo = WeiXinRequest.getUserInfoByCode(code, secret);
        FRUtils.FRLogInfo("userInfo:"+userInfo.toString());
        String decUserName = "";

        if (userInfo != null && userInfo.has("UserId")) {
            String userId = userInfo.optString("UserId");
            JSONObject detailUserInfo = WeiXinRequest.getDetailUserInfoByUserId(secret, userId);
            if (WeiXinRequest.isOk(detailUserInfo)) {
                FRUtils.FRLogInfo(String.format("userid为%s的用户详细信息为：", userId) + detailUserInfo.toString());
                if (WeiXinUserUtils.isWeiXinUserEnable(detailUserInfo)) {
                    WeiXinLoggerFactory.getLogger().debug("开始匹配平台用户。。。");
                    decUserName = WeiXinUserUtils.weiXinUserToDecUser(detailUserInfo);
                    FRUtils.FRLogInfo("decUserName");
                } else {
                    FRUtils.FRLogInfo("微信用户被禁用，请于微信管理后台设置！" + WeiXinLogErrorCode.WEIXIN_USER_DISABLE);
                }
            } else {
                FRUtils.FRLogInfo(String.format("获取userId为%s的用户详细信息失败！", userId) + WeiXinLogErrorCode.GET_USER_DETAIL_FAIL);
            }
        } else {
            FRUtils.FRLogInfo("code换取userId失败，请检查网络设置！" + WeiXinLogErrorCode.GET_USERID_FAIL);
        }

        return decUserName;
    }

}

