package com.fr.plugin.login.interception.filter;

import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.login.interception.IpInterceptionDBAccessProvider;
import com.fr.plugin.login.interception.dao.IpInterceptionDao;
import com.fr.plugin.login.interception.dao.UserReleaseDao;
import com.fr.plugin.login.interception.entity.IpInterceptionEntity;
import com.fr.plugin.login.interception.entity.UserReleaseEntity;
import com.fr.plugin.login.interception.function.CheckIpValid;
import com.fr.plugin.login.interception.function.MobileAgents;
import com.fr.plugin.login.utils.FRUtils;
import com.fr.stable.db.accessor.DBAccessor;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.restriction.RestrictionFactory;
import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

public class IpInterceptionFilter extends AbstractGlobalRequestFilterProvider {
    @Override
    public String filterName() {
        return "IpInterception";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{
                "/decision/*"
        };
    }

    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain filterChain){
        try {
            String allUrl = FRUtils.getAllUrl(req);
            if (isLogin(req)) {
                String currentUserName = LoginService.getInstance().getCurrentUserNameFromRequestCookie(req);
                String currentUserId = UserService.getInstance().getCurrentUserId(currentUserName);

                String userAgent = req.getHeader("User-Agent");
//                System.out.println(userAgent+"*******");
                FineLoggerFactory.getLogger().info("Login device : " + userAgent);

                //是不是移动端(移动端全部放xing)
                if (!MobileAgents.checkAgentIsMobile(req)) {
                    //是不是admin
                    if (!UserService.getInstance().isAdmin(currentUserId)) {
                        String ip = getIp(req);
                        FineLoggerFactory.getLogger().info("Is not admin at :" + ip);
                        FineLoggerFactory.getLogger().info("inAllowList :" + !checkUserInAllowList(currentUserName));
                        FineLoggerFactory.getLogger().info("checkUserCanLoginOnIp :" + !checkUserCanLoginOnIp(currentUserName, ip));

                        //是不是白名单
                        if (!checkUserInAllowList(currentUserName)) {
                            //是否可以在指定IP登陆
                            if (!checkUserCanLoginOnIp(currentUserName, ip)) {
//                                String warning = InterProviderFactory.getProvider().getLocText("Plugin-Dec_Filter_Warning_1")
//                                        + "\t" + ip + "\t"
//                                        + InterProviderFactory.getProvider().getLocText("Plugin-Dec_Filter_Warning_2");
//                                System.out.println(warning);
//                                req.setAttribute(DecisionServiceConstants.FINE_AUTH_TOKEN_NAME, "");

//                                WebUtils.printAsString(res, "You are not allow to login at " + ip + "5");
//                                Thread.sleep(1*1000);
//                                WebUtils.printAsString(res, "You are not allow to login at " + ip + "4");
//                                Thread.sleep(1*1000);
//                                WebUtils.printAsString(res, "You are not allow to login at " + ip + "3");
//                                Thread.sleep(1*1000);
//                                WebUtils.printAsString(res, "You are not allow to login at " + ip + "2");
//                                Thread.sleep(1*1000);
//                                WebUtils.printAsString(res, "You are not allow to login at " + ip + "1");
//                                Thread.sleep(1*1000);
//                                LoginService.getInstance().logout(req, res);
//                                new LogInOutResultInfo(req, res, currentUserName, false);
//                                WebUtils.printAsString(res, "You are not allow to login at " + ip);
//                                System.out.println(""+ isLogin(req));
//                                req.getRequestedSessionId();
//                                HttpSession httpSession = req.getSession();
//                                httpSession.invalidate();
//                                req.getAttributeNames();
//                                req.removeAttribute("");
//                                new LogInOutResultInfo(req, res, currentUserName, false);
//                                filterChain.doFilter(req, res);
//                                Logo;
                                LoginService.getInstance().logout(req, res);
//                                LoginService.getInstance().cleanLoginDetailInfo(5000);
//                                FineLoggerFactory.getLogger().info("\n-------------------" + isLogin(req));
//                                return ;
                            }
                        }
                    }else {
                        FineLoggerFactory.getLogger().info("Is admin");
                    }
                }else {
//                    FineLoggerFactory.getLogger().info("Login on Mobile");
                }
            }
            filterChain.doFilter(req, res);
//            LoginService.getInstance().logout(req, res);
        }catch (IOException | ServletException ignored){

        }catch (Exception e){
            e.printStackTrace();
        }
    }

    public static boolean checkUserCanLoginOnIp(String userName, String ip) throws Exception {
        DBAccessor dbAccessor = IpInterceptionDBAccessProvider.getDbAccessor();
        List<IpInterceptionEntity> entities = null;
        entities = dbAccessor.runQueryAction(daoContext ->
                daoContext.getDAO(IpInterceptionDao.class).find(
                        QueryFactory.create().addRestriction(RestrictionFactory.eq("userName", userName))
                )
        );
        /**
         * TODO: 1.登出 2.若填报页面内用户wei*，则适用于所有的用户
         */
        List<IpInterceptionEntity> allUser = dbAccessor.runQueryAction(daoContext ->
                daoContext.getDAO(IpInterceptionDao.class).find(
                        QueryFactory.create().addRestriction(RestrictionFactory.eq("userName", "*"))
                )
        );
//        entities.add(allUser.get());
        entities.addAll(allUser);
        boolean status = false;
        if (!entities.isEmpty()){
            String list = "";

            //只要循环里出现了1个true，就跳出循环
            for(int i = 0; i < entities.size() && status == false; i++){
                IpInterceptionEntity ipInterceptionEntity = entities.get(i);
//                if (ip.equals(ipInterceptionEntity.getStartIp())){
//                    status = true;
//                }
//                System.out.println("----------------"+ipInterceptionEntity.getEndIp());
                if (ipInterceptionEntity.getEndIp() == null || ipInterceptionEntity.getEndIp().isEmpty()){
                    status = CheckIpValid.ipIsValid(ipInterceptionEntity.getStartIp(), ipInterceptionEntity.getStartIp(), ip);
                    list = list.concat(ipInterceptionEntity.getStartIp()  + "\n");
                }else{
                    status = CheckIpValid.ipIsValid(ipInterceptionEntity.getStartIp(), ipInterceptionEntity.getEndIp(), ip);
                    list = list.concat(ipInterceptionEntity.getStartIp()  + " - " + ipInterceptionEntity.getEndIp() + "\n");
                }
//                if (status){
//                    i = entities.size() + 1;    //只要查出来一个true，就跳出循环
//                }
            }

            FineLoggerFactory.getLogger().info("\nAllow ip : " + list);
        }else {
            FineLoggerFactory.getLogger().info("Not found user in ip list!");
        }
//        return !entities.isEmpty();
        return status;
    }

    public static boolean checkUserInAllowList(String userName) throws Exception{
        DBAccessor dbAccessor = IpInterceptionDBAccessProvider.getDbAccessor();
        List<UserReleaseEntity> list = null;
        list = dbAccessor.runQueryAction(daoContext ->
                daoContext.getDAO(UserReleaseDao.class).find(
                        QueryFactory.create().addRestriction(RestrictionFactory.eq("userName", userName))
                )
        );
        FineLoggerFactory.getLogger().info("User in allow list : " + !list.isEmpty());
        return !list.isEmpty();
    }

    public static String getIp(HttpServletRequest req){

        /**
         * req.getRemoteAddr() : 默认的获取IP的方法，但是如果是多级代理则无法判断是否为真实IP
         * WebServiceUtils.getIpInfoFromRequest(req) : 帆软提供的获取IP的方法，不知道能不能在多级代理中获取真实IP
         */
//        return req.getRemoteAddr();
//        return WebServiceUtils.getIpInfoFromRequest(req);
        return CheckIpValid.getIp(req);
    }

    private boolean isLogin(HttpServletRequest req) {
        return LoginService.getInstance().isLogged(req);
    }

}
